# use Miniconda environment
#author = "Kang Yang"
#copyright = "Copyright 2017, Nanjing Greenland Hydrology Group"
#license = "GPL"
#version = "0.1"
#maintainer = "Kang Yang"
#email = "kangyang@nju.edu.cn"
#status = "Production"
#description = "plot lake/river information and all SMB models"

import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib
import pysal as ps
from matplotlib import cm
from textwrap import wrap

def create_all_SMB_DF(input_smb_file,area):
    data = pd.read_excel(input_smb_file)
    df = pd.DataFrame(data)
    df['DOY'] = df.index
    cols = list(df)
    mar_cols = [c for c in cols if 'mar' in c]
    racmo_cols = [c for c in cols if 'racmo' in c]
    merra_cols = [c for c in cols if 'merra' in c]

    mar_cols.insert(0,'DOY')
    racmo_cols.insert(0,'DOY')
    merra_cols.insert(0,'DOY')

    df_mar = df[mar_cols]
    df_racmo = df[racmo_cols]
    df_merra = df[merra_cols]
    df_mar.columns = ['DOY','<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']
    df_racmo.columns = ['DOY','<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']
    df_merra.columns = ['DOY','<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']
    df_mar.set_index('DOY', inplace=True)
    df_racmo.set_index('DOY', inplace=True)
    df_merra.set_index('DOY', inplace=True)

    df_mar = df_mar /(area * 1000000)*3600*1000  #chagne m3/s/day into mm/day, note, 24 hour day has been added previously
    df_racmo = df_racmo /(area * 1000000)*3600*1000
    df_merra = df_merra /(area * 1000000)*3600*1000

    return df_mar,df_racmo,df_merra

def get_eleBin_area(input_zonalSta_waterarea_point_file):
    dbf = ps.open(input_zonalSta_waterarea_point_file)
    d = dbf.by_col_array(dbf.header)
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    areaIdx = 'area'
    area = df.loc[areaIdx]

    return area

def get_axis_limits(ax, scale1=0.0,scale2=0.95):
    return ax.get_xlim()[1]*scale1, ax.get_ylim()[1]*scale2

def create_waterarea_fraction_DF(input_zonalSta_waterarea_point_file):
    dbf = ps.open(input_zonalSta_waterarea_point_file)
    d = dbf.by_col_array(dbf.header)
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    areaIdx = 'area'
    waterareaIdx = df.index[3:]

    area = df.loc[areaIdx]
    waterarea = df.loc[waterareaIdx]

    resolution = 30
    waterfraction = waterarea*resolution*resolution/(1000000*area.values)
    waterfraction.index = [int(day) for day in waterareaIdx]
    waterfraction = waterfraction.sort_index(ascending=True)
    return waterfraction

def create_watervolume_DF(input_zonalSta_watervolume_point_file):
    dbf = ps.open(input_zonalSta_watervolume_point_file)
    d = dbf.by_col_array(dbf.header)
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    areaIdx = 'area'
    waterdepthIdx = df.index[3:]

    area = df.loc[areaIdx]
    waterdepth = df.loc[waterdepthIdx]

    resolution = 30
    overestimation_rate = 0.8  #according to Pope, the spectral method overestimates water, use this coefficient to rectify the volume
    waterVolume = waterdepth * resolution * resolution / (1000000*area.values) * 1000 * overestimation_rate# water volume in mm
    waterVolume.index = [int(day) for day in waterdepthIdx]
    waterVolume = waterVolume.sort_index(ascending=True)
    return waterVolume

def create_SMB_DF(input_SMBExcelFile):
    filename, ext = os.path.splitext(input_SMBExcelFile)
    if ext == '.xlsx':
        data = pd.read_excel(input_SMBExcelFile)
    if ext == '.csv':
        data = pd.read_csv(input_SMBExcelFile)
    df = pd.DataFrame(data)
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']
    if ext == '.xlsx':
        days = df.index[:]
    if ext == '.csv':
        days = df.index[1:-1]  #In csv files, the first index is 'basin', which needs to be deleted
        df = df.iloc[1:-1]
        
    df.index = [int(day[3:]) for day in days]
    print df
    return df

def create_river_lake_DF(river_lake_stat_file):

    dbf = ps.open(river_lake_stat_file)
    d = dbf.by_col_array(dbf.header)  # my solution is better than the last sentence
    df = pd.DataFrame(d)
    df.columns = dbf.header
    df = df.T
    df.columns = ['<1000 m','1000-1200 m', '1200-1400 m','1400-1600 m','1600-1800 m','>1800 m']

    types = df.index[:]
    areaIdx = 'area'
    lakeareaIdx = [ t for t in types if 'Larea' in t]
    lakecountIdx = [ t for t in types if 'Lcount' in t]
    riverIdx = [ t for t in types if 'Rlength' in t]

    #Use .iloc for integer based indexing and .loc for label based indexing.
    area = df.loc[areaIdx]
    lakearea = df.loc[lakeareaIdx]
    lakecount = df.loc[lakecountIdx]
    riverlength = df.loc[riverIdx]

    lakearea_dayIdx = lakearea.index[:]
    lakearea_dayIdx = [int(day[:3]) for day in lakearea_dayIdx]
    river_dayIdx = riverlength.index[:]
    river_dayIdx = [int(day[:3]) for day in river_dayIdx]

    lakefraction = lakearea/area.values
    drainage_density = riverlength/area.values

    lakecount.index = lakearea_dayIdx
    lakefraction.index = lakearea_dayIdx
    drainage_density.index = river_dayIdx

    lakecount = lakecount.sort_index(ascending=True)
    lakefraction = lakefraction.sort_index(ascending=True)
    drainage_density = drainage_density.sort_index(ascending=True)

    return lakecount,lakefraction,drainage_density

def select_doy_Runoff_from_all(df_runoff,df_all_runoff):
    doys = df_runoff.index[:]
    df_runoff = df_all_runoff.loc[doys]
    return df_runoff

def plot_scatter_1(df_runoff,df_all_runoff,waterfraction,watervolume,riverfraction,lakefraction):

    #set plotting environments
    matplotlib.style.use('classic')
    matplotlib.rcParams.update({'font.size': 40})
    matplotlib.rcParams['axes.linewidth'] = 2
    matplotlib.rcParams['xtick.major.size'] = 40
    matplotlib.rcParams['xtick.major.width'] = 2
    matplotlib.rcParams['ytick.major.size'] = 40
    matplotlib.rcParams['ytick.major.width'] = 2
    matplotlib.rcParams['lines.linewidth'] = 3
    matplotlib.rcParams['lines.markersize'] = 300
    f, ax1 = plt.subplots()
    #f, axarr = plt.subplots(2,2)
    #Google: How to use colormaps to color plots of Pandas DataFrames
    #also see colormaps_reference
    #also see Invert colormap in matplotlib
    #cmap = cm.get_cmap('bwr_r') # Colour map (there are many others)

    color=iter(cm.bwr_r(np.linspace(0,1,8)))

    df_runoff = select_doy_Runoff_from_all(df_runoff,df_all_runoff)
    #df_runoff = df_runoff.cumsum()
    doys = df_runoff.index[:]

    i = 0
    c = next(color)
    for column in waterfraction:
        if i == 3:
            c = next(color)  #to match colors
        ax1.scatter(df_runoff[column],waterfraction[column],s=15*(df_runoff.index-145),marker='o',c=c)
        c = next(color)
        i = i + 1

    ax1.set_ylabel('Meltwater fraction')
    ax1.set_xlabel('SMB-modeled daily surface runoff (mm)')
    ax1.set_ylim([-0.02, 0.1])
    ax1.set_xlim([-5, 25])
    ax1.legend(loc='upper right',fontsize=30)

    #ax2 = ax1.twinx() 
    #waterfraction.plot(ax=ax2,legend=False,linestyle="None",marker = 'o',cmap=cmap)
    #ax2.set_ylabel('water area fraction')
    #ax2.annotate('(b)',size = 20,xy=get_axis_limits(ax2))
    #ax2.set_ylim([0, 0.08])
    #ax2.set_yticks(np.arange(0,0.1,0.02))
    #ax2.set_xlim([160, 240])

    plt.show()

    f.savefig(r'E:\scatter_merra_runoff_vs_water_fraction.tif', format='tif', dpi=450)

def plot_final(df_runoff,df_all_runoff,df_racmo,df_merra,waterfraction,watervolume,riverfraction,lakefraction):
    f, axarr = plt.subplots(5,2)

    #f, axarr = plt.subplots(5,2,sharex=True)
    #f, axarr = plt.subplots(2,2)
    #Google: How to use colormaps to color plots of Pandas DataFrames
    #also see colormaps_reference
    #also see Invert colormap in matplotlib
    cmap = cm.get_cmap('bwr_r') # Colour map (there are many others)
    df_all_runoff.plot(ax=axarr[0][0],legend=False,cmap=cmap)
    df_runoff.plot(ax=axarr[0][0],legend=False,linestyle="None",marker = 'o',cmap=cmap)
    #axarr[0][0].set_xlabel('DOY')
    axarr[0][0].set_ylabel('MAR R (mm)')
    axarr[0][0].annotate('(a)',size = 20,xy=get_axis_limits(axarr[0][0]))
    axarr[0][0].set_ylim([0, 70])
    axarr[0][0].set_xlim([160, 240])


    df_all_runoff = df_all_runoff.cumsum()
    df_acc_runoff = select_doy_Runoff_from_all(df_runoff,df_all_runoff)
    df_all_runoff.plot(ax=axarr[0][1],legend=False,cmap=cmap)
    df_acc_runoff.plot(ax=axarr[0][1],legend=False,linestyle="None",marker = 'o',cmap=cmap)
    #axarr[0][0].set_xlabel('DOY')
    axarr[0][1].set_ylabel('\n'.join(wrap('accumulative MAR R (mm)',13)))
    axarr[0][1].annotate('(a)',size = 20,xy=get_axis_limits(axarr[0][0]))
    axarr[0][1].set_ylim([0, 600])
    axarr[0][1].set_yticks(np.arange(0,800,200))
    axarr[0][1].set_xlim([160, 240])

    df_racmo.plot(ax=axarr[1][0],legend=False,cmap=cmap)
    df_runoff = select_doy_Runoff_from_all(df_runoff,df_racmo)
    df_runoff.plot(ax=axarr[1][0],legend=False,linestyle="None",marker = 'o',cmap=cmap)
    #axarr[0][0].set_xlabel('DOY')
    axarr[1][0].set_ylabel('RACMO R (mm)')
    axarr[1][0].annotate('(a)',size = 20,xy=get_axis_limits(axarr[1][0]))
    axarr[1][0].set_ylim([0, 70])
    axarr[1][0].set_xlim([160, 240])

    df_racmo = df_racmo.cumsum()
    df_acc_runoff = select_doy_Runoff_from_all(df_runoff,df_racmo)
    df_racmo.plot(ax=axarr[1][1],legend=False,cmap=cmap)
    df_acc_runoff.plot(ax=axarr[1][1],legend=False,linestyle="None",marker = 'o',cmap=cmap)
    #axarr[0][0].set_xlabel('DOY')
    axarr[1][1].set_ylabel('\n'.join(wrap('accumulative RACMO R (mm)',13)))
    axarr[1][1].annotate('(a)',size = 20,xy=get_axis_limits(axarr[1][1]))
    axarr[1][1].set_ylim([0, 600])
    axarr[1][1].set_yticks(np.arange(0,800,200))
    axarr[1][1].set_xlim([160, 240])

    df_merra.plot(ax=axarr[2][0],legend=False,cmap=cmap)
    df_runoff = select_doy_Runoff_from_all(df_runoff,df_merra)
    df_runoff.plot(ax=axarr[2][0],legend=False,linestyle="None",marker = 'o',cmap=cmap)
    axarr[2][0].set_xlabel('DOY')
    axarr[2][0].set_ylabel('MERRA R (mm)')
    axarr[2][0].annotate('(a)',size = 20,xy=get_axis_limits(axarr[2][0]))
    #axarr[1][0].set_ylim([0, 60])
    axarr[2][0].set_xlim([160, 240])

    df_merra = df_merra.cumsum()
    df_acc_runoff = select_doy_Runoff_from_all(df_runoff,df_merra)
    df_merra.plot(ax=axarr[2][1],legend=False,cmap=cmap)
    df_acc_runoff.plot(ax=axarr[2][1],legend=False,linestyle="None",marker = 'o',cmap=cmap)
    axarr[2][1].set_xlabel('DOY')
    axarr[2][1].set_ylabel('\n'.join(wrap('accumulative MERRA R (mm)',13)))
    axarr[2][1].annotate('(a)',size = 20,xy=get_axis_limits(axarr[2][1]))
    axarr[2][1].set_ylim([0, 600])
    axarr[2][1].set_yticks(np.arange(0,800,200))
    axarr[2][1].set_xlim([160, 240])

    # --------------- lake river -----------------------#

    waterfraction.plot(ax=axarr[3][0],legend=False,marker = 'o',cmap=cmap)
    #axarr[1][0].set_xlabel('DOY')
    axarr[3][0].set_ylabel('water fraction')
    axarr[3][0].annotate('(c)',size = 20,xy=get_axis_limits(axarr[1][0]))
    axarr[3][0].set_ylim([0, 0.08])
    axarr[3][0].set_yticks(np.arange(0,0.1,0.02))

    watervolume.plot(ax=axarr[3][1],legend=False,marker = 'o',cmap=cmap)
    #axarr[1][1].set_xlabel('DOY')
    axarr[3][1].set_ylabel('\n'.join(wrap('water storage (mm)',14)))
    axarr[3][1].annotate('(d)',size = 20,xy=get_axis_limits(axarr[2][0]))
    axarr[3][1].set_ylim([0, 160])
    axarr[3][1].set_yticks(np.arange(0,180,40))


    riverfraction.plot(ax=axarr[4][0],legend=False,marker = 'o',cmap=cmap)
    axarr[4][0].set_xlabel('DOY')
    axarr[4][0].set_ylabel('river fraction')
    axarr[4][0].annotate('(d)',size = 20,xy=get_axis_limits(axarr[1][1]))
    axarr[4][0].set_ylim([0, 0.08])
    axarr[4][0].set_yticks(np.arange(0,0.1,0.02))

    lakefraction.plot(ax=axarr[4][1],legend=False,marker = 'o',cmap=cmap)
    axarr[4][1].set_xlabel('DOY')
    axarr[4][1].set_ylabel('lake fraction')
    axarr[4][1].annotate('(d)',size = 20,xy=get_axis_limits(axarr[2][1]))
    axarr[4][1].set_ylim([0, 0.08])
    axarr[4][1].set_yticks(np.arange(0,0.1,0.02))

    axarr[4][1].legend(loc='upper left',fontsize=12)

    plt.show()

    f.savefig(r'E:\seasonal_evolution_2015.tif', format='tif', dpi=450)

#set plotting environments
matplotlib.style.use('classic')
matplotlib.rcParams.update({'font.size': 18})
matplotlib.rcParams['axes.linewidth'] = 2
matplotlib.rcParams['xtick.major.size'] = 16
matplotlib.rcParams['xtick.major.width'] = 2
matplotlib.rcParams['ytick.major.size'] = 16
matplotlib.rcParams['ytick.major.width'] = 2
matplotlib.rcParams['lines.linewidth'] = 3
matplotlib.rcParams['lines.markersize'] = 10


#output of step 7
zonalSta_waterarea_point_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\zonalstats\ele_pt_waterarea.dbf'

#output of step 11
river_lake_stat_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\1_statistics\river_lake_statistics.dbf'

#output of step 13
smbPath = r'E:\1.GrIS_river_evolution\1_SMB_outputs\2015'
all_runoffFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_runoff_output.csv'
all_meltFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_melt_output.csv'
all_nd_runoffFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_nondelayed_runoff_output.csv'


#output of step 14
runoffFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_runoff_output.xlsx'
meltFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_melt_output.xlsx'
nd_runoffFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_nondelayed_runoff_output.xlsx'

#ouput of step 15
zonalSta_watervolume_point_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\zonalstats\ele_pt_watervolume.dbf'


lakecount,lakefraction,drainage_density = create_river_lake_DF(river_lake_stat_file)
waterfraction = create_waterarea_fraction_DF(zonalSta_waterarea_point_file)
riverfraction = waterfraction - lakefraction
riverfraction[riverfraction<0]=0
df_runoff = create_SMB_DF(runoffFile)
df_melt = create_SMB_DF(meltFile)
df_nd_runoff = create_SMB_DF(nd_runoffFile)
df_nd_runoff[df_nd_runoff<0]=0


df_all_runoff = create_SMB_DF(all_runoffFile)
df_all_melt = create_SMB_DF(all_meltFile)
df_all_nd_runoff = create_SMB_DF(all_nd_runoffFile)
df_all_nd_runoff[df_all_nd_runoff<0]=0

watervolume = create_watervolume_DF(zonalSta_watervolume_point_file)

# run E:\1.GrIS_river_evolution\00_codes\lake_filling_vs_SMB_runoff_tempuse, step 1-4 to get runoff for all SMB models
# note run SMB_Step4_calculate_smb_runoff_for_specific_days_new.py!! 2018.03.29
input_smb_file = r'E:\1.GrIS_river_evolution\00_codes\lake_filling_vs_SMB_runoff_tempuse\output_smb_runoff_output_for_DOYs.xlsx'


#output of step 13
smbPath = r'E:\1.GrIS_river_evolution\1_SMB_outputs\2015'
all_runoffFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_runoff_output.csv'
all_meltFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_melt_output.csv'
all_nd_runoffFile = smbPath + '\\mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_nondelayed_runoff_output.csv'
#output of step 14
runoffFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_runoff_output.xlsx'
nd_runoffFile = smbPath + '\\DOYs_mar_7_5km_lat_lon_ps_basin_Watson_Isortoq_cells_nondelayed_runoff_output.xlsx'
df_runoff = create_SMB_DF(runoffFile)
df_all_runoff = create_SMB_DF(all_runoffFile)

#output of step 7, only for area
zonalSta_waterarea_point_file = r'E:\1.GrIS_river_evolution\water_mask\2015\Watson_Isortoq\zonalstats\ele_pt_waterarea.dbf'
area = get_eleBin_area(zonalSta_waterarea_point_file)

df_mar,df_racmo,df_merra = create_all_SMB_DF(input_smb_file,area)


print watervolume


plot_final(df_runoff,df_all_runoff,df_racmo,df_merra,waterfraction,watervolume,riverfraction,lakefraction)

#plot_scatter_1(df_runoff,df_merra,waterfraction,watervolume,riverfraction,lakefraction)